/*
 * gc global variable provides access to GUI Composer infrastructure components and project information.
 * For more information, please see the Working with Javascript guide in the online help.
 */
var gc = gc || {};
gc.services = gc.services || {};

/*
 *  Boilerplate code for encoding and decoding data transmitted to / received from TI devices.
 */

var targetAccessErrorHandler = function (error) {
  if (error && error.message) {
    throw error.message;
  } else {
    throw error;
  }
};

var count = 0;

/**
 * This code implements the IPacketCodec interface.
 *
 * @constructor
 * @implements gc.databind.IPacketCodec
 */
CustomCodec = function () {};

var initCodec = function () {

  if (!gc || !gc.databind || !gc.databind.IPollingPacketCodec) {
    console.log("called initCodec but stuff is undefined");
    return;
  }

  // use default implementations of connect,disconnect,encode and decode functions:
  CustomCodec.prototype = new gc.databind.IPollingPacketCodec();

  CustomCodec.prototype.writeCmdTimer = null;
  CustomCodec.prototype.maxBitWidth = 16;
  CustomCodec.prototype.readCmdSymbols = null;
  CustomCodec.prototype.writeCmdSymbols = null;
  CustomCodec.prototype.pendingWriteCmds = null;
  CustomCodec.prototype.refreshListener = null;
  CustomCodec.prototype.preRefreshListener = null;
  CustomCodec.prototype.skipReadsUntilNextReadCmdFlagSet = false;
  CustomCodec.prototype.preRefreshTimeInMs = 30;
  CustomCodec.prototype.regBlocks = null;

  CustomCodec.prototype.setReadCmdFlag = function (reg) {
    if (reg) {
      var varsObj = {};
      varsObj['ManReadAddr'] = reg.addr;
      varsObj[reg.parentGroup.structReadCmdName] = 1;
      updateProcessQueue('W', reg.parentGroup.deviceObj, varsObj);
    } else {
    }
  };

  CustomCodec.prototype.writeValue = function (symbol, value) {

    gc.console.log("CustomCodec", "CustomCodec.writeValue(" + symbol.regName + "," + value + ")");
    var deviceObj = "";
    if (symbol.parentGroup && symbol.parentGroup.deviceObj) {
      deviceObj = symbol.parentGroup.deviceObj;
    }

    var varsObj = {};
    varsObj['ManWriteAddr'] = symbol.addr;
    varsObj['ManWriteData'] = value;
    varsObj[symbol.parentGroup.structWriteCmdName] = 1;
    return updateProcessQueue('W', deviceObj, varsObj);
  };

  CustomCodec.prototype.readValue = function (symbol) {
    var structName = "";
    if (symbol.parentGroup && symbol.parentGroup.structName) {
      structName = symbol.parentGroup.structName + "."
    }
    var origValue = +symbol.value;
    if ((CustomCodec.prototype.pendingWriteCmds && CustomCodec.prototype.pendingWriteCmds.length > 0) || (CustomCodec.prototype.skipReadsUntilNextReadCmdFlagSet)) {
      gc.console.log("CustomCodec", "CustomCodec.readValue deferred for " + symbol.regName + " due to pending write.");
      return Q(origValue);
    }
    gc.console.debug("CustomCodec", "CustomCodec.readValue(" + symbol.regName + ")");
    structName = structName + symbol.regName;
    return updateProcessQueue('R', structName);
  };

  CustomCodec.prototype.onPreRefresh = function (oldValue, newValue, progress) {
    if (this.pendingWriteCmds && this.pendingWriteCmds.length > 0) {
      gc.console.log("CustomCodec", "skipping setting readCmd flag because of pending write(s)")
    } else {
      gc.console.debug("CustomCodec", "preRefresh: set ReadCmd flag");
      this.setReadCmdFlag();
    }
  };

  CustomCodec.prototype.onRefresh = function (oldValue, newValue, progress) {
    gc.console.log("CustomCodec", "refresh");
  };

  CustomCodec.prototype.connect = function (settings) {
    this.model = document.querySelector('ti-model-register');
    if (this.model) {
      var modelId = this.model.id;
      var refreshBind = gc.databind.registry.getBinding(modelId + '.$refresh_interval');
      if (refreshBind) {
        refreshBind.setPreRefreshInterval(this.preRefreshTimeInMs);
        this.preRefreshListener = new gc.databind.IChangedListener();
        this.preRefreshListener.onPreRefresh = this.onPreRefresh.bind(this);
        refreshBind.addPreRefreshListener(this.preRefreshListener);

        this.refreshListener = new gc.databind.IChangedListener();
        this.refreshListener.onRefresh = this.onRefresh.bind(this);
        refreshBind.addRefreshListener(this.refreshListener);

      }
      this.model.getRegisterViewInformation(0).then(function (deviceObj) {
        this.maxBitWidth = deviceObj.info.regsize;
        this.readCmdSymbols = [];
        this.regBlocks = deviceObj.regblocks;
        for (var i = 0; i < deviceObj.regblocks.length; i++) {
          var regBlock = deviceObj.regblocks[i];
          var structName = "";
          if (regBlock.structName && regBlock.structName.length > 0) {
            structName = regBlock.structName + ".";
          }
          if (regBlock.structReadCmdName) {
            var readCmdSymbol = structName + regBlock.structReadCmdName;
            this.readCmdSymbols.push(readCmdSymbol);
          }
          if (regBlock.structWriteCmdName) {
            var writeCmdSymbol = structName + regBlock.structWriteCmdName;
            this.writeCmdSymbols.push(writeCmdSymbol);
          }
        }
      }.bind(this));
    }
    var regPage = document.querySelector('#ti_widget_register_page');
    if (regPage) {
      regPage.preReadDelay = this.preRefreshTimeInMs;
      regPage.addEventListener('pre_read_reg', function (event) {
        this.setReadCmdFlag(event.detail.reg);
      }.bind(this));
    }

  };

  // registerCustomCodec has to be called right at startup in order for the codec to be recognized and used
  gc.console.log("CustomCodec", "CustomCodec: about to register...");
  var baseCodecs;
  gc.databind.registerCustomCodec("customRegModelCodec", CustomCodec, baseCodecs);
}


var cmdInProgress = false;
var commQueue = [];

function convertToRegObj(val) {
  let regNameArr = ['fault_status_reg', 'diag_status1_reg', 'diag_status2_reg', 'diag_status3_reg', 'ctrl1_reg', 'ctrl2_reg', 'ctrl3_reg', 'ctrl4_reg', 'ctrl5_reg', 'ctrl6_reg', 'ctrl7_reg', 'ctrl8_reg', 'ctrl9_reg', 'ctrl10_reg', 'ctrl11_reg', 'ctrl12_reg', 'ctrl13_reg', 'index1_reg', 'index2_reg', 'index3_reg', 'index4_reg', 'index5_reg', 'custom_ctrl1_reg', 'custom_ctrl2_reg', 'custom_ctrl3_reg', 'custom_ctrl4_reg', 'custom_ctrl5_reg', 'custom_ctrl6_reg', 'custom_ctrl7_reg', 'custom_ctrl8_reg', 'custom_ctrl9_reg', 'atq_ctrl1_reg', 'atq_ctrl2_reg', 'atq_ctrl3_reg', 'atq_ctrl4_reg', 'atq_ctrl5_reg', 'atq_ctrl6_reg', 'atq_ctrl7_reg', 'atq_ctrl8_reg', 'atq_ctrl9_reg', 'atq_ctrl10_reg', 'atq_ctrl11_reg', 'atq_ctrl12_reg', 'atq_ctrl13_reg', 'atq_ctrl14_reg', 'atq_ctrl15_reg', 'atq_ctrl16_reg', 'atq_ctrl17_reg', 'atq_ctrl18_reg', 'ss_ctrl1_reg', 'ss_ctrl2_reg', 'ss_ctrl3_reg', 'ss_ctrl4_reg', 'ss_ctrl5_reg', 'ctrl14_reg'];
  let regObj = {};
  for (var i = 0; i < val.length; ++i) {
    regObj[regNameArr[i]] = val[i];
  }
  return regObj;
}

setInterval(function () {
  if (!cmdInProgress) {
    if (commQueue.length > 0) {
      cmdInProgress = true;
      var thisCmd = commQueue.shift();
      if (thisCmd.processType === 'R') {
        var regName = thisCmd.structName.split('.').pop();
        gc.target.access.readMemory(0x6300, 'unsigned char', 55).then(function (val) {
          if (Object.keys(val).includes("indexedErrors") && Object.keys(val.indexedErrors).length > 0) {
            thisCmd.promise.reject(`Target failed to read 0x6300`);
          } else {
            val = convertToRegObj(val["values"]);
            //customLogMessage(thisCmd.value.ManWriteAddr, val, 'R');
            if (Object.keys(val).includes(regName)) {
              thisCmd.promise.resolve(val[regName]);
            } else {
              // console.log(`${regName} is not present in gDeviceSpiReg`);
              thisCmd.promise.reject(`${regName} is not present in gDeviceSpiReg`);
            }
            if (commQueue.length > 0) {
              var writeRegItemIndex = commQueue.findIndex((item) => item.processType === 'W');
              writeRegItemIndex = writeRegItemIndex === -1 ? commQueue.length : writeRegItemIndex;
              for (var i = 0; i < writeRegItemIndex; i++) {
                thisCmd = commQueue.shift();
                regName = thisCmd.structName.split('.').pop();
                if (Object.keys(val).includes(regName)) {
                  thisCmd.promise.resolve(val[regName]);
                } else {
                  // console.log(`${regName} is not present in gDeviceSpiReg`);
                  thisCmd.promise.reject(`${regName} is not present in gDeviceSpiReg`);
                }
              }
            }
          }
          cmdInProgress = false;
        }).fail(function (err) {
          thisCmd.promise.reject(err);
          cmdInProgress = false;
          // commQueue = [];
        });
      } else if (thisCmd.processType === 'W') {
        gc.target.access.writeValue(thisCmd.structName, thisCmd.value).then(function (val) {
          thisCmd.promise.resolve();
          cmdInProgress = false;
        }).fail(function (err) {
          thisCmd.promise.reject(err);
          cmdInProgress = false;
          // commQueue = [];
        });
      }
    }
  }
}, 10);


function updateProcessQueue(processType, processObj, updateVal) {
  var deferred = Q.defer();
  commQueue.push({
    promise: deferred,
    processType: processType,
    structName: processObj,
    value: updateVal
  });
  return deferred.promise;
}

var templateObj = document.querySelector('#template_obj');
if (templateObj) {
  initCodec();
} else {
  document.addEventListener('WebComponentsReady', initCodec);
}